//Arduino Uno with Matrix Orbital GTT50A-TPR-BLM-B0-H1-CS-V5 and BBC
//Created by Clark, 27/07/2016
//support@matrixorbital.ca
//www.matrixorbital.ca/appnotes

#include <Wire.h>
#include <stdlib.h>

//global variables
#define I2C_Address 0x28  //Define default 8bit I2C address of 0x50 >> 1 for 7bit Arduino
#define ASCII_Format 0    //Define ASCII label format
#define Label_Index 1     //Define label index
#define Code_Length 4     //Deine length of the code

char key_code[Code_Length];
char input_code[Code_Length];
int input_index;
int printable_chars;

void setup() {
  //Setup I2C bus
  Wire.begin();
  delay(500);
  input_index = 0;
  printable_chars = 1;
  //Set Key Code (1234) and Input Code (0000)
  for (int i = 0; i < Code_Length; i++) {
    key_code[i] = i + 49;
    input_code[i] = 48;
  }
  resetDisplay();
  //Wait for display to reset
  delay(3000);
}

void loop() {
  byte data = i2cRead();  //Read the command prefix
  switch (data) {
    case 252:
      data = i2cRead();  //Read the touch prefix
      switch (data) {
        case 135:  //Handle touch message
          int message_length = i2cRead() << 8 | i2cRead();  //Read mesage length
          byte event = i2cRead();   //Read event type
          byte id = i2cRead();  //Read region that generated the touch response
          switch (id) {
            case 10: {
                delay(250);
                if (checkCode()) {
                  activateBuzzer(1000, 500);
                  setLabelFontColour(Label_Index, 0, 255, 0); //Set label colour to green
                }
                else {
                  activateBuzzer(100, 250);
                  setLabelFontColour(Label_Index, 255, 0, 0); //Set label colour to red
                }
              }
              break;
            case 11: {
                clearCode();
                updateLabel(Label_Index, ASCII_Format, input_code, 0);
              }
              break;
            default: {
                input_code[input_index] = id + 48;
                setLabelFontColour(Label_Index, 0, 0, 0);
                updateLabel(Label_Index, ASCII_Format, input_code, printable_chars);
                updateIndices();
              }
              break;
          }
          break;
      }
      break;
  }
}

bool checkCode() {
  bool correct_code = true;
  for (int i = 0; i < Code_Length; i++) {
    if (input_code[i] != key_code[i]) {
      correct_code = false;
    }
  }
  return correct_code;
}

void clearCode() {
  for (int i = 0; i < Code_Length; i++) {
    input_code[i] = 32;
  }
  input_index = 0;
  printable_chars = 1;
}

void updateIndices() {
  input_index = input_index + 1;
  if (input_index >= Code_Length) {
    input_index = 0;
  }
  printable_chars = printable_chars + 1;
  if (printable_chars > Code_Length) {
    printable_chars = 4;
  }
}

void resetDisplay() {
  char command[] = { 254, 1 };
  i2cWrite(command, sizeof(command));
}

void activateBuzzer(short frequency, short duration) {
  char command[] = { 254, 187, (byte)(frequency >> 8), (byte)(frequency), (byte)(duration >> 8), (byte)(duration)};
  i2cWrite(command, sizeof(command));
}

void setLabelFontColour(byte index, byte red, byte green, byte blue) {
  char command[] = { 254, 21, index, red, green, blue};
  i2cWrite(command, sizeof(command));
  //Response will be handled by loop()
}

void updateLabel(byte index, byte format, char* message, byte message_length) {
  char command[] = { 254, 17, index, format };
  i2cWrite(command, sizeof(command));
  i2cWrite(message, message_length);
  char terminator[] = { 0 };
  i2cWrite(terminator, 1);
}

void i2cWrite(char* data, byte data_length) { //Write an arraty of bytes over i2c
  Wire.beginTransmission(I2C_Address);
  for (int i = 0; i < data_length; i++) {
    Wire.write(data[i]);
  }
  Wire.endTransmission();
}

byte i2cRead() { //Wait for one byte to be read over i2c
  do {
    Wire.requestFrom(I2C_Address, 1);
  } while (Wire.available() < 1);
  return (Wire.read());
}
